////////////////////////////////////////////////////////////////////
///
///   Copyright (C) 2008 Scalingweb.com
///   All rights reserved. 
/// 
///   Author: Yuriy Soroka <ysoroka@scalingweb.com>
///
////////////////////////////////////////////////////////////////////

#include "MorkParser.h"
#include <string.h>
#include <iostream>
#include <cstring>
#include <cstdlib>
using namespace std;

// import from string_case_find.cpp:
bool string_case_find(const string& str1, const string& str2);
// TODO: REPLACE WITH strcasestr (NOT AVAILABLE IN MINGW)

#if !(defined(WINDOWS) || defined(WIN32) || defined(_WIN32))
	// import from itoa.c:
	void itoa(int value, char* str, int base);
#endif

// Address book params

const char constFirstName[]		= "FirstName";
const char constLastName[]		= "LastName";
const char constNickName[]		= "NickName";
const char constPrimaryEmail[]	= "PrimaryEmail";
const char constHomePhone[]		= "HomePhone";
const char constFaxNumber[]		= "FaxNumber";
const char constCellularNumber[]= "CellularNumber";
const char constWorkPhone[]		= "WorkPhone";
const char constHomeAddress[]	= "HomeAddress";
const char constHomeAddress2[]	= "HomeAddress2";
const char constHomeCity[]		= "HomeCity";
const char constHomeState[]		= "HomeState";
const char constHomeZipCode[]	= "HomeZipCode";
const char constHomeCountry[]	= "HomeCountry";
const char constWorkAddress[]	= "WorkAddress";
const char constWorkAddress2[]	= "WorkAddress2";
const char constWorkCity[]		= "WorkCity";
const char constWorkState[]		= "WorkState";
const char constWorkZipCode[]	= "WorkZipCode";
const char constWorkCountry[]	= "WorkCountry";
const char constJobTitle[]		= "JobTitle";
const char constDepartment[]	= "Department";
const char constCompany[]		= "Company";
const char constWebPage1[]		= "WebPage1";
const char constWebPage2[]		= "WebPage2";
const char constNotes[]			= "Notes";

class AddressBookEntry
{
	public:

		/// Entry ID
		std::string id;

		/// Name
		std::string first_name;
		std::string last_name;
		std::string nick_name;

		/// Telephones/Faxes
		std::string home_tel;
		std::string mobile_tel;
		std::string work_tel;
		std::string fax;

		/// Addresses
		std::string address_work;
		std::string address_home;

		/// Web Page
		std::string web_page;

		/// Email
		std::string email;

		/// Notes
		std::string notes;
};

typedef std::map< int, AddressBookEntry > AbeMap;
typedef std::map< std::string, std::string > RawAbeMap;

AbeMap abes_;


void fromRawAbe( RawAbeMap &rawAbe, std::string &retText, const char *paramTitle )
{
	RawAbeMap::iterator iter;
	iter = rawAbe.find( paramTitle );

	if ( iter != rawAbe.end() )
	{
		retText = std::string( iter->second.c_str() );		
	}
}

inline void appendAddress( std::string &text, const std::string &add )
{
	if ( !add.empty() )
	{
		text += add;
		text += ", ";
	}
}

void addEntry( RawAbeMap &rawAbe, AddressBookEntry &abe )
{
	fromRawAbe( rawAbe, abe.first_name, constFirstName );
	fromRawAbe( rawAbe, abe.last_name, constLastName );
	fromRawAbe( rawAbe, abe.nick_name, constNickName );
	fromRawAbe( rawAbe, abe.email, constPrimaryEmail );

	fromRawAbe( rawAbe, abe.fax, constFaxNumber );
	fromRawAbe( rawAbe, abe.mobile_tel, constCellularNumber );
	fromRawAbe( rawAbe, abe.home_tel, constHomePhone );
	fromRawAbe( rawAbe, abe.work_tel, constWorkPhone );
	fromRawAbe( rawAbe, abe.notes, constNotes );

	fromRawAbe( rawAbe, abe.web_page, constWebPage1 );

	// Build addresses
	std::string HomeAddress;
	fromRawAbe( rawAbe, HomeAddress, constHomeAddress );
	std::string HomeAddress2;
	fromRawAbe( rawAbe, HomeAddress2, constHomeAddress2 );
	std::string HomeCity;
	fromRawAbe( rawAbe, HomeCity, constHomeCity );
	std::string HomeState;
	fromRawAbe( rawAbe, HomeState, constHomeState );
	std::string HomeZipCode;
	fromRawAbe( rawAbe, HomeZipCode, constHomeZipCode );
	std::string HomeCountry;
	fromRawAbe( rawAbe, HomeCountry, constHomeCountry );

	std::string WorkAddress;
	fromRawAbe( rawAbe, WorkAddress, constWorkAddress );
	std::string WorkAddress2;
	fromRawAbe( rawAbe, WorkAddress2, constWorkAddress2 );
	std::string WorkCity;
	fromRawAbe( rawAbe, WorkCity, constWorkCity );
	std::string WorkState;
	fromRawAbe( rawAbe, WorkState, constWorkState );
	std::string WorkZipCode;
	fromRawAbe( rawAbe, WorkZipCode, constWorkZipCode );
	std::string WorkCountry;
	fromRawAbe( rawAbe, WorkCountry, constWorkCountry );
	std::string JobTitle;
	fromRawAbe( rawAbe, JobTitle, constJobTitle );
	std::string Department;
	fromRawAbe( rawAbe, Department, constDepartment );
	std::string Company;
	fromRawAbe( rawAbe, Company, constCompany );

	std::string address;

	appendAddress( address, HomeAddress );
	if ( !address.empty() ) address += "\r\n";
	appendAddress( address, HomeAddress2 );
	if ( !HomeAddress2.empty() ) address += "\r\n";
	appendAddress( address, HomeCity );
	appendAddress( address, HomeState );
	appendAddress( address, HomeZipCode );
	if ( !HomeCity.empty() || !HomeState.empty() || !HomeZipCode.empty() ) address += "\r\n";
	appendAddress( address, HomeCountry );

	abe.address_home = address;

	address.clear();

	appendAddress( address, JobTitle );
	appendAddress( address, Department );
	if ( !address.empty() ) address += "\r\n";
	appendAddress( address, Company );
	if ( !Company.empty() ) address += "\r\n";
	appendAddress( address, WorkAddress );
	if ( !WorkAddress.empty() ) address += "\r\n";
	appendAddress( address, WorkAddress2 );
	if ( !WorkAddress2.empty() ) address += "\r\n";
	appendAddress( address, WorkCity );
	appendAddress( address, WorkState );
	appendAddress( address, WorkZipCode );
	if ( !WorkCity.empty() || !WorkState.empty() || !WorkZipCode.empty() ) address += "\r\n";
	appendAddress( address, WorkCountry );
	if ( !WorkCountry.empty() ) address += "\r\n";

	abe.address_work = address;
}



///
/// Opens address book 
/// @param path - path to the address book file
/// @return - true if success, otherwise false
///
bool openAddressBook( const std::string& path )
{
	abes_.clear();
	MorkParser mork;

	// Open and parse mork file
	if ( !mork.open( path ) )
	{
		return false;
	}

	const int defaultScope = 0x80;

	MorkTableMap *Tables = 0;
	MorkRowMap *Rows = 0;
	MorkTableMap::iterator tableIter;
	MorkRowMap::iterator rowIter;

	Tables = mork.getTables( defaultScope );

	if ( Tables )
	{
		// Iterate all tables
		for ( tableIter = Tables->begin(); tableIter != Tables->end(); tableIter++ )
		{
			if ( 0 == tableIter->first ) continue;

			// Get rows
			Rows = mork.getRows( defaultScope, &tableIter->second );

			if ( Rows )
			{
				// Iterate all rows
				for ( rowIter = Rows->begin(); rowIter != Rows->end(); rowIter++ )
				{
					if ( 0 == rowIter->first ) continue;

					RawAbeMap ram;
					AddressBookEntry abe;
					std::string column;
					std::string value;
					
					char buffer[20];
					itoa( rowIter->first, buffer, 10 );

					abe.id = std::string( buffer );

					// Get cells
					for ( MorkCells::iterator cellsIter = rowIter->second.begin();
						cellsIter != rowIter->second.end(); cellsIter++ )
					{
						column = mork.getColumn( cellsIter->first );
						value = mork.getValue( cellsIter->second );

						ram[ column ] = value;
					}

					AbeMap::iterator abeIter;

					abes_[ rowIter->first ] = abe;
					abeIter = abes_.find( rowIter->first );

					addEntry( ram, abeIter->second );
				}
			}
		}
	}

	return true;
}


int main( int argc, char ** argv )
{
	if(argc<=2)
	{
		std::cout << "usage: mab [mab file] [search query]" << endl;
		return 1;
	}

	if( false == openAddressBook( argv[1] ) )
	{
		std::cout << "unable to open " << argv[1] << endl;
		return 1;
	}

	AbeMap::iterator iter;
	int matches = 0;
	for ( iter = abes_.begin(); iter != abes_.end(); iter++ )
	{
		string first_name = iter->second.first_name;
		string last_name = iter->second.last_name;
		string nick_name = iter->second.nick_name;
		string home_tel = iter->second.home_tel;
		string mobile_tel = iter->second.mobile_tel;
		string work_tel = iter->second.work_tel;
		string fax = iter->second.fax;
		string address_home = iter->second.address_home;
		string address_work = iter->second.address_work;
		string web_page = iter->second.web_page;
		string email = iter->second.email;
		string notes = iter->second.notes;
		
		if( 
			string_case_find(first_name, string(argv[2])) ||
			string_case_find(last_name, string(argv[2])) ||
			string_case_find(nick_name, string(argv[2])) ||
			string_case_find(home_tel, string(argv[2])) ||
			string_case_find(mobile_tel, string(argv[2])) ||
			string_case_find(work_tel, string(argv[2])) ||
			string_case_find(fax, string(argv[2])) ||
			string_case_find(email, string(argv[2])) ||
			string_case_find(notes, string(argv[2]))
		  )
		{
			std::cout << " ---------------------------------------------- " << endl;
			std::cout << "Name: " << first_name << " " << last_name << endl;
			if(!nick_name.empty()) std::cout << "Nickname: " << nick_name << endl;
			if(!home_tel.empty()) std::cout << "Home telephone: " << home_tel << endl;
			if(!mobile_tel.empty()) std::cout << "Mobile telephone: " << mobile_tel << endl;
			if(!work_tel.empty()) std::cout << "Work telephone:" << work_tel << endl;
			if(!work_tel.empty()) std::cout << "FAX:" << fax << endl;
			if(!address_home.empty()) std::cout << "Home address: " << address_home << endl; // TODO: keep indentation
			if(!address_work.empty()) std::cout << "Work address: " << address_work << endl; // TODO: keep indentation
			if(!web_page.empty()) std::cout << "Website: " << web_page << endl;
			if(!email.empty()) std::cout << "E-mail: " << email << endl;
			// fix newlines in notes string before printing
				{
					int n=0;
					while((n=notes.find("10"))!=string::npos)
						notes.replace(n, 2, "\n");
				}
			if(!notes.empty()) std::cout << "Notes: " << notes << endl;  // TODO: keep indentation
			std::cout << " ---------------------------------------------- " << endl;
			//std::cout << endl;
			//std::cout << endl;
			matches++;
		}
	} // ENDOFR
	
	cout << argv[0] << ": " << matches << " matches found" << endl;
	
	return 0;
}
